import numpy as np
import torch
from logger import Logger
LOGGER = Logger().logger()
device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
from discriminator import Discriminator
from generator import Generator
from loss import Loss
from preprocess import LoadData
from tqdm import tqdm
from utils import initialize_weights
class Train():
    def __init__(self):
        '''
        @params
        @return
        '''
        self.gen_XY = Generator().to(device)
        self.gen_YX = Generator().to(device)
        self.dis_X = Discriminator().to(device)
        self.dis_Y = Discriminator().to(device)
        self.gen_XY.apply(initialize_weights)
        self.gen_YX.apply(initialize_weights)
        self.dis_X.apply(initialize_weights)
        self.dis_Y.apply(initialize_weights)

        self.gen_XY_optim = torch.optim.Adam(self.gen_XY.parameters(), lr=0.0002, betas=(0.5, 0.999))
        self.dis_X_optim = torch.optim.Adam(self.dis_X.parameters(), lr=0.0002, betas=(0.5, 0.999))
        self.dis_Y_optim = torch.optim.Adam(self.dis_Y.parameters(), lr=0.0002, betas=(0.5, 0.999))

        self.gen_XY_scheduler = torch.optim.lr_scheduler.StepLR(self.gen_XY_optim, step_size=100, gamma=0.1)

        self.dis_X_scheduler = torch.optim.lr_scheduler.StepLR(self.dis_X_optim, step_size=100, gamma=0.1)
        self.dis_Y_scheduler = torch.optim.lr_scheduler.StepLR(self.dis_Y_optim, step_size=100, gamma=0.1)

        self.cycle_loss = Loss().cycle_loss.to(device)
        self.identity_loss = Loss().identity_loss.to(device)
        self.adversarial_loss = Loss().adversarial_loss.to(device)

        self.losses = {"G": [], "D": [], "C": [], "I": [], "T": []}

    def train():
        '''
        @params
        @return
        '''
        EPOCHS = 200
        for epoch in range(EPOCHS):
            '''
            Steps:
            For Generator
            1. Load Images
            2. Reset Gradients
            3. Generate Fake Images
            4. Calculate Advetisarial Loss
            5. Generate Original Images from Fake Images
            6. Calculate Cycle Consistency Loss
            7. Total Loss
            8. Backpropagate
            9. Update Weights

            For Discriminator
            10. Reset Gradients
            11. Calculate Adversarial Loss between labels for real images and generated images
            12. Total loss = Average of both losses
            13. Backpropagate
            14. Update Weights
            15. Do same for other discriminator

            16. Update Logs
            17. Save Image
            18. Update Learning Rate

            '''